import Flutter
import LoyaltyCore
import Terra

public class SwiftLoyaltyFlutterPlugin: NSObject, FlutterPlugin, FlutterLoyaltyApi {
    public func getInstanceAppName(_ appName: String, error: AutoreleasingUnsafeMutablePointer<FlutterError?>) -> CommonResult? {
        let result = CommonResult()
        do {
            let _ = try getTerraloyalty(appName: appName)
            result.isSuccess = true
        } catch {
            result.error = error.localizedDescription
        }
        
        return result
        
    }
    
    public func getConfigurationAppName(_ appName: String, completion: @escaping (FlutterLoyaltyConfigurationResult?, FlutterError?) -> Void) {
        var result = FlutterLoyaltyConfigurationResult()
        do {
            let terraLoylaty = try getTerraloyalty(appName: appName)
            result = convertLoyaltyConfiguration(config: terraLoylaty.config)
        } catch {
            result.isSuccess = false
            result.error = error.localizedDescription
        }
        completion(result, nil)
    }

    
    public func registerMemberAppName(_ appName: String, request: FlutterRegisterMemeberRequest, completion: @escaping (FlutterRegisterMemberResult?, FlutterError?) -> Void) {
        do {
            let terraLoylaty = try getTerraloyalty(appName: appName)
            terraLoylaty.register(param: convertLoyaltyRegisterMemberRequest(request: request)) { registerMemberResult in
                switch registerMemberResult {
                case .success(let registerMember):
                    let result = convertRegisterMemberResult(registerMemberResult: registerMember)
                    completion(result, nil)
                case .failure(let error):
                    let result = FlutterRegisterMemberResult()
                    result.isSuccess = false
                    result.error = error.localizedDescription
                    completion(result, nil)
                }
            }
        } catch {
            let result = FlutterRegisterMemberResult()
            result.isSuccess = false
            result.error = error.localizedDescription
            completion(result, nil)
        }
        
    }
    
    public func getMembershipQrAppName(_ appName: String, completion: @escaping (FlutterMembershipQrResult?, FlutterError?) -> Void) {
        let errorResult = FlutterMembershipQrResult()
        errorResult.isSuccess = false
        do {
            let terraLoylaty = try getTerraloyalty(appName: appName)
            terraLoylaty.getEncryptedCode { encryptedCodeResult in
                switch encryptedCodeResult {
                case .success(let encryptedCodeResponse):
                    guard let qrData = encryptedCodeResponse.result?.encryptedMembershipQrdata else {
                        errorResult.error = "Loyalty API Error"
                        completion(errorResult, nil)
                        return
                    }
                    
                    let result = convertMembershipQrResult(encryptedMemCode: qrData)
                    completion(result, nil)
                case .failure(let error):
                    errorResult.error = error.errorDescription
                    completion(errorResult, nil)
                }
            }
            
        } catch {
            errorResult.error = "Loyalty API Error"
            completion(errorResult, nil)
        }
                                          }
    
    public func getMemberInfoAppName(_ appName: String, request: FlutterMemberInfoRequest, completion: @escaping (FlutterMemberInfoResult?, FlutterError?) -> Void) {
        do {
            let terraLoylaty = try getTerraloyalty(appName: appName)
            
            
            terraLoylaty.getMemberInfo(convertMemberInfoType(request: request)) { memberInfoResult in
                switch memberInfoResult {
                case .success(let memberInfoResponse):
                    let result = convertMemberInfoResult(memberInfo: memberInfoResponse)
                    completion(result, nil)
                case .failure(let error):
                    let result = FlutterMemberInfoResult()
                    result.isSuccess = false
                    result.error = error.localizedDescription
                    completion(result, nil)
                }
           
            }
        } catch {
            let result = FlutterMemberInfoResult()
            result.isSuccess = false
            result.error = error.localizedDescription
            completion(result, nil)
        }
    }
    
    public func getTransactionHistoryAppName(_ appName: String, request: FlutterTransactionHistoryRequest, completion: @escaping (FlutterTransactionHistoryResult?, FlutterError?) -> Void) {
        do {
            let terraLoylaty = try getTerraloyalty(appName: appName)
            terraLoylaty.getHistory(params: convertTransactionHistoryRequeset(request: request)) { historyResponseResult in
                switch historyResponseResult {
                case .success(let historyResponse):
                    let result = convertTransactionHistoryResult(transactionHistoryResult: historyResponse)
                    completion(result, nil)
                case .failure(let error):
                    let result = FlutterTransactionHistoryResult()
                    result.isSuccess = false
                    result.error = error.localizedDescription
                    completion(result, nil)
                }
            }
          
        } catch {
            let result = FlutterTransactionHistoryResult()
            result.isSuccess = false
            result.error = error.localizedDescription
            completion(result, nil)
        }
    }
    
    public func getNetworkConfigAppName(_ appName: String, completion: @escaping (FlutterNetworkConfigResult?, FlutterError?) -> Void) {
        do {
            let terraLoylaty = try getTerraloyalty(appName: appName)
            terraLoylaty.getNetworkConfig{ networkConfigResult in
                switch networkConfigResult {
                case .success(let networkConfig):
                    let flutterResult = convertNetworkConfig(networkConfigResponse: networkConfig)
                    completion(flutterResult, nil)
                case .failure(let error):
                    let flutterResult = FlutterNetworkConfigResult()
                    flutterResult.isSuccess = false
                    flutterResult.error = error.localizedDescription
                    completion(flutterResult, nil)
                }
            }
        } catch {
            let flutterResult = FlutterNetworkConfigResult()
            flutterResult.isSuccess = false
            flutterResult.error = error.localizedDescription
            completion(flutterResult, nil)
        }
    }
    
    public func getClientIdAppName(_ appName: String, completion: @escaping (FlutterClientIdResult?, FlutterError?) -> Void) {
        let flutterResult = FlutterClientIdResult()
        do {
            let terraLoylaty = try getTerraloyalty(appName: appName)
            flutterResult.isSuccess = true
            flutterResult.clientId = terraLoylaty.getClientId()
        } catch {
            flutterResult.isSuccess = false
            flutterResult.error = error.localizedDescription
        }
        completion(flutterResult, nil)
    }
    
    public func assignMemberCardIdAppName(_ appName: String, memberId: String, memberCardId: String, completion: @escaping (CommonResult?, FlutterError?) -> Void) {
        let flutterResult = CommonResult()
        do {
            let terraLoylaty = try getTerraloyalty(appName: appName)
            terraLoylaty.assignMemberCardId(memberId: memberId, memberCardId: memberCardId) { assignMemberCardIdResult in
                switch assignMemberCardIdResult {
                case .success():
                    flutterResult.isSuccess = true
                    completion(flutterResult, nil)
                case .failure(let error):
                    flutterResult.isSuccess = false
                    flutterResult.error = error.localizedDescription
                    completion(flutterResult, nil)
                }
            }
        } catch {
            flutterResult.isSuccess = false
            flutterResult.error = error.localizedDescription
            completion(flutterResult, nil)
        }
    }
    
    private func getTerraloyalty(appName: String) throws -> Loyalty {
        let terraApp = try TerraApp.getInstance(appName: appName)
        
        guard let terraLoyalty = TerraLoyalty.getInstance(by: terraApp) else {
            throw LoyaltyException.createInstanceError
        }
        return terraLoyalty
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let flutterLoyaltyApi = SwiftLoyaltyFlutterPlugin.init()
        FlutterLoyaltyApiSetup(registrar.messenger(), flutterLoyaltyApi)
    }
}


