import 'package:pigeon/pigeon.dart';

@HostApi()
abstract class FlutterLoyaltyApi {
  CommonResult getInstance(
    String appName,
  );

  @async
  FlutterLoyaltyConfigurationResult getConfiguration(String appName);

  @async
  FlutterRegisterMemberResult registerMember(
    String appName,
    FlutterRegisterMemeberRequest request,
  );

  @async
  FlutterMembershipQrResult getMembershipQr(String appName);

  @async
  FlutterMemberInfoResult getMemberInfo(
    String appName,
    FlutterMemberInfoRequest request,
  );

  @async
  FlutterTransactionHistoryResult getTransactionHistory(
    String appName,
    FlutterTransactionHistoryRequest request,
  );

  @async
  FlutterNetworkConfigResult getNetworkConfig(String appName);

  @async
  FlutterClientIdResult getClientId(String appName);

  @async
  CommonResult assignMemberCardId(String appName, String memberId, String memberCardId);
}

// Model
class CommonResult {
  bool? isSuccess;
  String? error;
}

class FlutterLoyaltyConfigurationResult {
  bool? isSuccess;
  String? error;
  FlutterLoyaltyConfiguration? loyaltyConfiguration;
}

class FlutterLoyaltyConfiguration {
  String? linkMembershipUrl;
  String? registerMemberUrl;
  bool? allowLinkMembership;
  bool? allowRegisterMembership;
  String? membershipCodeFormat;
}

class FlutterRegisterMemeberRequest {
  String? phone;
  String? name;
  String? address;
  String? dateOfBirth;
  String? email;
  FlutterGender? gender;
  String? idCardNumber;
  String? idCitizenNumber;
  String? passportNumber;
}

enum FlutterGender { female, male, none }

class FlutterRegisterMemberResult {
  bool? isSuccess;
  String? error;
  FlutterRegisterMember? registerMember;
}

class FlutterRegisterMember {
  String? memberId;
}

class FlutterMembershipQrResult {
  bool? isSuccess;
  String? error;
  FlutterMembershipQr? membershipQr;
}

class FlutterMembershipQr {
  String? code;
  int? issueAt;
  int? expireAt;
}

class FlutterMemberInfoRequest {
  String? phone;
  String? memberQrCode;
}

class FlutterMemberInfoResult {
  bool? isSuccess;
  String? error;
  FlutterMemberInfo? flutterMemberInfo;
}

class FlutterMemberInfo {
  String? memberQrCode;
  String? memberId;
  String? phone;
  String? memberCardId;
  String? name;
  bool? isActive;
  int? point;
  int? tierPoint;
  int? keepTierUntil;
  int? accumulationFrom;
  int? accumulationTo;
  String? currentTierCode;
  String? currentTierName;
  int? currentTierMinPoint;
  String? nextTierName;
  int? nextTierMinPoint;
  FlutterCardType? defaultCardType;
}

enum FlutterCardType { qr, memberCard }

class FlutterTransactionHistoryRequest {
  String? merchantCode;
  String? fromTime;
  String? toTime;
  List<String?>? txnTypes;
  FlutterStatus? status;
  int? page;
  int? pageSize;
}

enum FlutterTransactionType {
  grant,
  spend,
  returnGrant,
  returnSpend,
  initiate,
  initiatePromotion,
  issue
}

class FlutterTransactionHistoryResult {
  bool? isSuccess;
  String? error;
  FlutterTransactionHistory? transactionHistory;
}

class FlutterTransactionHistory {
  List<FlutterTransaction?>? transactions;
  FlutterPageData? pageData;
}

class FlutterTransaction {
  String? id;
  String? refId;
  String? orderCode;
  FlutterTransactionType? txnType;
  String? txnTypeText;
  String? txnDesc;
  FlutterAccountingType? accountingType;
  FlutterStatus? status;
  int? point;
  int? createdAt;
  String? createdBy;
  int? networkId;
  String? networkName;
  String? merchantCode;
  String? merchantName;
  String? terminalCode;
  String? terminalName;
  String? terminalAddress;
}

class FlutterPageData {
  int? currentPage;
  int? totalPage;
  int? pageSize;
  int? totalItems;
}

enum FlutterAccountingType { debit, credit }

enum FlutterStatus {
  init,
  initFailure,
  noAccouning,
  accountingSuccess,
  accountingFailure,
  accouting,
}

class FlutterNetworkConfigResult {
  bool? isSuccess;
  String? error;
  FlutterNetworkData? networkConfig;
}

class FlutterNetworkData {
  FlutterNetwork? networkInfo;
  FlutterNetworkConfig? networkConfig;
  List<FlutterTier?>? networkTiers;
}

class FlutterNetwork {
  String? id;
  String? name;
}

class FlutterNetworkConfig {
  int? exchangeRate;
  String? currencyImg;
  String? currencyName;
  String? cardDefaultImg;
  String? avatarDefaultImg;
  bool? allowToGetMemberByPhone;
  String? faqWebUrl;
  String? benefitWebUrl;
  String? brandColor;
  bool? allowMerchantIssueMemberCard;
}

class FlutterTier {
  String? code;
  String? name;
  int? fromPoint;
  String? cardTieredImg;
  String? tierIconImg;
}

class FlutterClientIdResult {
  bool? isSuccess;
  String? error;
  String? clientId;
}
